//Geoget2 category library


//..............................................................................
//podle knihovny Category upravil Gord tak, aby bylo mozne ukladat souradnice
//a vyhazet body, ktere pekracuji limit

//..............................................................................
//Authors:
//Geby
//Medwyn
//Gord


//..............................................................................
//Changelog:
// v01 - Initial release (Category)
// 0.2 - category_coord (Gord)

{$ifndef DEBUG_HELPER}
unit category_coord;
{$endif}

type
  TCatCallback = procedure(const geo: tgeo; const wpt: TWpt; category: string);
  
var
  cat_db: TSqliteDatabase;
  cat_callback: TCatCallback;
  cat_break:boolean;
  
//Starts new transaction
procedure CatBeginUpdate;
begin
  cat_db.Start('','');
end;

//Commits the transaction
procedure CatEndUpdate;
begin
  cat_db.Commit('');
end;

//Adds geocache to category database
procedure CatAddGC(GC: TGeo; category,lat,lon: string);
begin
  cat_db.AddParamInt(':pkey', GC.Key);
  cat_db.AddParamInt(':ptab', 0);
  cat_db.AddParamText(':cat', category);
  cat_db.AddParamText(':lat', lat);
  cat_db.AddParamText(':lon', lon);
  cat_db.ExecSQL('INSERT INTO cat (ptrkey, ptrtab, category,lat, lon) VALUES (:pkey, :ptab, :cat, :lat, :lon)');
end;

//Adds additional waypoint to category database
procedure CatAddWpt(Wpt: TWpt; category,lat,lon: string);
begin
  cat_db.AddParamInt(':pkey', Wpt.Key);
  cat_db.AddParamInt(':ptab', 1);
  cat_db.AddParamText(':cat', category);
  cat_db.AddParamText(':lat', lat);
  cat_db.AddParamText(':lon', lon);
  cat_db.ExecSQL('INSERT INTO cat (ptrkey, ptrtab, category, lat, lon) VALUES (:pkey, :ptab, :cat, :lat, :lon)');
end;

//Sorts the database and for each point calls the Cat_Callback method.
procedure CatSort(maxPoints:integer);
var
  tab: TSqliteTable;
  geo: TGeo;
  wpt: TWpt;
  x: integer;
  category: string;
  key: int64;
  aLatLonCnt,col,colVal,colCnt:string;
  rowId:integer;
begin
  cat_break:=false;
  if(cat_callback=nil) then exit;
  //##### Gord - doplneni
  //pro kazdou kategorii zjistime seznam bodu, kterych je na stejne souradnicich
  //vic nez je povoleno (maxPoints)
  cat_db.Start('Redukce','');
  cat_db.AddParamInt(':maxPoints', maxPoints);
  col:='category||"; "||lat||"; "||lon';
  tab:=cat_db.GetTable('SELECT '+col+' latlon, COUNT('+col+') cnt FROM cat '
		+'GROUP BY '+col+' HAVING count('+col+')>:maxPoints',false);
  try
    while not tab.EOF do begin
      aLatLonCnt:=aLatLonCnt+tab.FieldAsString(tab.FieldIndex['latlon'])+'#'+tab.FieldAsString(tab.FieldIndex['cnt'])+'#';
      tab.Next();
      if(GeoBusyTest()) then begin
        cat_break:=true;
        break;
      end;
    end;
  finally
    tab.Free();
  end;
  //najdeme body se stejnymi souradnicemi a prebytecne vymazeme
  while (aLatLonCnt<>'') do begin
    if((GeoBusyTest()) or (cat_break)) then begin
      cat_break:=true;
      break;
    end;
    colVal:=Fetch(aLatLonCnt,'#');
    colCnt:=Fetch(aLatLonCnt,'#');
    cat_db.AddParamText(':latlon',colVal);
    tab:=cat_db.GetTable('SELECT rowid FROM cat WHERE '+col+'=:latlon ORDER BY rowid;',false);
    try
      x:=0;
      while not tab.EOF do begin
        if((GeoBusyTest()) or (cat_break)) then begin
          cat_break:=true;
          break;
        end;
        Inc(x);
        if(x>maxPoints) then begin
          rowId:=tab.FieldAsInteger(tab.FieldIndex['rowid']);
          cat_db.AddParamInt(':rowid',rowId);
          cat_db.AddParamText(':col',colVal);
          cat_db.ExecSQL('DELETE FROM cat WHERE '+col+'=:col AND rowid>=:rowid;');
          break; //dal uz nepokracujeme, vsechny prebyvajici jsme smazali
        end;
        tab.Next();
      end;
    finally
      tab.Free();
    end;
  end;
  cat_db.Commit('Redukce');
  //##### Gord - konec doplneni

  geo := TGeo.Create;
  wpt := TWpt.Create;
  try
    tab := cat_db.GetTable('SELECT ptrkey,ptrtab,category FROM cat ORDER BY category', false);
    try
      while not tab.EOF do begin
        if(GeoBusyTest()) then begin
          cat_break:=true;
          break;
        end;
        key := tab.FieldAsInteger(tab.FieldIndex['ptrkey']);
        x := tab.FieldAsInteger(tab.FieldIndex['ptrtab']);
        category := tab.FieldAsString(tab.FieldIndex['category']);
//LDH('XXXXX key='+IntToStr(key)+', x='+IntToStr(x)+', '+category);
        if x = 0 then begin //je to keska
          geo.LoadByKey(key);
          Cat_Callback(geo, nil, category);
        end
        else begin          //je to wpt
          wpt.LoadByKey(key);
          Cat_Callback(geo, wpt, category);
        end;
        tab.next;
      end;
    finally
      tab.free;
    end;
  finally
    wpt.free;
    geo.free;
  end;
end;

procedure catInit;
begin
  cat_callback := nil;
  cat_db := TSqliteDatabase.Create(''); //bez jmena souboru se vytvari tabulka v pameti
  cat_db.ExecSQL('PRAGMA page_size = 4096');
  cat_db.ExecSQL(
    //'CREATE TABLE cat (key INTEGER PRIMARY KEY, ' //pokud je PRIMARY KEY, nema tabulka rowid
    'CREATE TABLE cat (key INTEGER'
    + ',ptrkey INTEGER'
    + ',ptrtab INTEGER'
    + ',category TEXT'
    + ',lat TEXT'
    + ',lon TEXT'
    //+ ',idx INTEGER AUTOINCREMENT'
    +')'
    );
  cat_db.ExecSQL('CREATE INDEX keyidx ON cat (key)');
  cat_db.ExecSQL('CREATE INDEX catidx ON cat (category)');
  cat_db.ExecSQL('CREATE INDEX latlon ON cat (category||"; "||lat||"; "||lon)');
end;

//Returns categories in category database. 
procedure CatCategories(const value: TStrings);
begin
  cat_db.GetTableStrings('SELECT distinct(category) FROM cat ORDER BY category', value);
end;

//Returns total size of category database - total count of points
function CatSize: Integer;
begin
 Result := cat_db.GetTableValue('SELECT count(*) FROM cat');
end;

//Returns count of points in given category
function CatCategorySize(category : string) : Integer;
begin
   cat_db.AddParamText(':cat',category);
 Result := cat_db.GetTableValue('SELECT count(*) FROM cat WHERE category LIKE :cat'); 
end;

//Frees up the category database. 
procedure catFinish;
begin
  cat_db.free;
end;


{$ifndef DEBUG_HELPER}
end.
{$endif}
